/*
 * CGI countdown timer - main routines
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include "library.h"
#include "hash.h"
#include "rope.h"
#include "cgi.h"
#include "smalloc.h"
#include "log.h"

/* Resource directory on server for images */
#define DIR_LOCATION	DIR_RESOURCE "/countdown/images/"

/* Fingers out from here on */
#define STREQ(x, y) (strcmp(x, y) == 0)

#define TIME_MILLENIUM	946684800

int get_digit(char *str);

typedef
struct _cd_info_t {
	char	*timezone;
	char	*digitset;
	int		 digit;
	time_t	 target;
} cd_info_t;

void
countdown_show(cd_info_t *info)
{
	time_t		 now;
	time_t		 interval;

	int			 days;
	int			 hours;
	int			 minutes;

	int			 val;
	char		*server;

	server = getenv("SERVER_NAME");
	if (!server) server = DEFAULT_SERVER;

	time(&now);
	interval = info->target - now;
	if (interval < 0) interval = 0;

	interval /= 60;						/* Work in minutes */
	days = interval / (24 * 60);		/* Integer arithmetic */
	interval %= (24 * 60);				/* Get remaining minutes */
	hours = interval / 60;				/* Integers again */
	minutes = interval % 60;			/* And the rest */

	switch(info->digit) {
		case 0: val = minutes % 10;		break;	/* Minute units */
		case 1: val = minutes / 10;		break;	/* Minute tens */
		case 2: val = hours % 10;		break;	/* Hour units */
		case 3: val = hours / 10;		break;	/* Hour tens */
		case 4: val = days % 10;		break;	/* Day units */
		case 5: val = (days/10)    %10;	break;	/* Day tens */
		case 6: val = (days/100)   %10;	break;	/* Day hundreds */
		case 7: val = (days/1000)  %10;	break;	/* Day thousands */
		case 8: val = (days/10000) %10;	break;	/* Day lots */
		case 9: val = (days/100000)%10;	break;	/* Day bignums */
		default:	val = 0;	/* Defeat compile warnings */
	}

	printf("Location: http://%s" DIR_LOCATION "%s/%c.gif" "\n\n",
					server, info->digitset, ((char)(val + '0')) );
}

/* QUERY_STRING format: Traditional CGI a=b&c=d
 * Parameters:
 * case				- A funny, meaningless parameter
 * digitPosition	- digit to show, may be (hours|minutes)(tens|units)
 * All other parameters are ignored
 */
void
countdown_info_long(cd_info_t *info)
{
	hash_t	*cgihash;
	char	*temp;

	cgihash = cgi_hash_new();

/* Get the digit */
	temp = hash_get(cgihash, "digitPosition");
	if (!temp) {
		log("cgiconf", PRI_CRIT, "No 'digitPosition' in QUERY_STRING "
						"for long format");
		exit(0);
	}
	info->digit = get_digit(temp);
	if (info->digit == -1) {
		log("cgiconf", PRI_CRIT, "digitPosition parameter `%s' is "
						"invalid", temp);
		exit(0);
	}

	info->digitset = "default";

	temp = hash_get(cgihash, "target");
	if (temp)
		info->target = atol(temp);
/* OW hackish */
	else if ( (temp = hash_get(cgihash, "case")) )
		info->target = TIME_MILLENIUM;
	else {
		log("cgiconf", PRI_CRIT, "No 'target' in QUERY_STRING "
						"for long format");
		exit(0);
	}
}

/* QUERY_STRING format:
 * %c%c%s:%s
 * %c - digitset
 * %c - digit (ASCII value mod 7 is used)
 * %s - target
 * :%s  - literal ':' followed by any string - optional
 */
void
countdown_info_short(cd_info_t *info)
{
	rope_t	*cgirope;
	char	*temp;
	char	*d;
	int		 i;

	/* OK, so we allocate twice. Hell. */
	cgirope = cgi_rope_new();
	temp = rope_chars(cgirope);

	i = strcspn(temp, ":");
	if (i < 2) {
		log("cgisys", PRI_CRIT, "strlen(QUERY_STRING) < 2 for new "
						"protocol");
		exit(0);
	}
	temp[i] = 0;					/* Modify rope by reference */
	d = temp++;						/* Points to rope */
	/* Let's base this on '0' ASCII */
	info->digit = ((unsigned char)(*temp)) + 700 - '0';
	info->digit %= 10;
	*temp++ = 0;					/* Terminate digitset as a string */
	info->digitset = d;				/* Points to rope */
	info->target = atol(temp);		/* Still points into rope */
}

void
invoke(void)
{
	rope_t		*cgirope;
	cd_info_t	 ci;

	cgirope = cgi_rope_new();

	/* Decide which protocol we are on */
	if (strchr(rope_chars(cgirope), '&')) {
		countdown_info_long(&ci);
	}
	else {
		countdown_info_short(&ci);
	}

	countdown_show(&ci);
}
