%{
 * CGI email client - template parser
 * Copyright (C) 1999 Open World Ltd
 * Written by Shevek <shevek@anarres.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>
#include <unistd.h>
#include <errno.h>
#include "library.h"
#include "config.h"
#include "cgi.h"
#include "hash.h"
#include "rope.h"
#include "log.h"

void parser_field_print(char *str);

hash_t	*yyhash;

%}

%option nounput
%option noyywrap

tVALIDCHAR	[a-zA-Z_]
tFIELD		\{{tVALIDCHAR}+\}

%%

{tFIELD}		{	parser_field_print(yytext);					}

%%

void
parser_field_print(char *str)
{
	char	*temp;
	char	*data;

	temp = strchr(++str, '}');
	if (temp) *temp = 0;	/* ASSERTED by lex anyhow */

	data = hash_get(yyhash, str);
	if (data)
		fprintf(yyout, "%s", data);
	else if ( (data = getenv(str)) )
		fprintf(yyout, "%s", data);
	else
		fprintf(yyout, "Field %s was not present in hash table", str);
}

FILE *
parser_stream(rope_t *rope)
{
	FILE	*input;

	input = fopen(rope_chars(rope), "r");
	if (!input) {
		log("cgiconf", PRI_CRIT, "Could not open template '%s': %s",
						rope_chars(rope), strerror(errno));
		exit(0);
	}

	rope_delete(rope);

	return input;
}

void
parser_parse(hash_t *cgihash, rope_t *rope, FILE *output)
{
	yyin = parser_stream(rope);
	yyhash = cgihash;

	if (output)
		yyout = output;

	printf("Content-type: text/html\n\n");
	yylex();
}
